<?php
namespace Ak\Wp\Init;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Laminas\Diactoros\Response\RedirectResponse;
use Laminas\Diactoros\Uri;

/**
 * Class BaseUrl
 *
 * @package Ak\Wp\Init
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp
 */
final class BaseUrl implements MiddlewareInterface
{
    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $uri = $request->getUri();
        $uriPath = $uri->getPath();
        $baseUrl = $this->detectBaseUrl($request);

        if (!empty($baseUrl) && strpos($uriPath, $baseUrl) === 0) {
            $path = substr($uriPath, strlen($baseUrl));
            $path = sprintf('/%s', ltrim($path, '/'));
            /** @noinspection PhpUndefinedMethodInspection */
            $request = $request->withUri($uri->withPath($path))
                ->withAttribute('base_url', $baseUrl);
        }

        wp_urls($request);

        $response = $handler->handle($request);

        if ($response instanceof RedirectResponse) {
            $redirectUri = new Uri($response->getHeaderLine('Location'));
            $redirectUri = $redirectUri->withPath($baseUrl . $redirectUri->getPath());

            $response = $response->withHeader('Location', (string) $redirectUri);
        }

        return $response;
    }

    /**
     * Pokušava otkriti koji dio putanje requesta je stvaran.
     *
     * @param ServerRequestInterface $request
     *
     * @return string
     */
    public function detectBaseUrl(ServerRequestInterface $request = null): string
    {
        $server = $request ? $request->getServerParams() : [];

        $filename = $server['SCRIPT_FILENAME'] ?? ($_SERVER['SCRIPT_FILENAME'] ?? '');
        $scriptName = $server['SCRIPT_NAME'] ?? ($_SERVER['SCRIPT_NAME'] ?? '');
        $phpSelf = $server['PHP_SELF'] ?? ($_SERVER['PHP_SELF'] ?? '');
        $origScriptName = $server['ORIG_SCRIPT_NAME'] ?? ($_SERVER['ORIG_SCRIPT_NAME'] ?? '');

        if ($scriptName !== null && basename($scriptName) === $filename) {
            $baseUrl = $scriptName;
        } elseif ($phpSelf !== null && basename($phpSelf) === $filename) {
            $baseUrl = $phpSelf;
        } elseif ($origScriptName !== null && basename($origScriptName) === $filename) {
            // 1and1 shared hosting compatibility.
            $baseUrl = $origScriptName;
        } else {
            // Backtrack up the SCRIPT_FILENAME to find the portion
            // matching PHP_SELF.

            $baseUrl  = '/';
            $basename = basename($filename);
            if ($basename) {
                $path     = ($phpSelf ? trim($phpSelf, '/') : '');
                $basePos  = strpos($path, $basename) ?: 0;
                $baseUrl .= substr($path, 0, $basePos) . $basename;
            }
        }

        // If the baseUrl is empty, then simply return it.
        if (empty($baseUrl)) {
            return '';
        }

        // Does the base URL have anything in common with the request URI?
        $requestUri = $server['REQUEST_URI'] ?? ($_SERVER['REQUEST_URI'] ?? '');

        // Full base URL matches.
        if (0 === strpos($requestUri, $baseUrl)) {
            return dirname($baseUrl);
        }

        // Directory portion of base path matches.
        $baseDir = str_replace('\\', '/', dirname($baseUrl));
        if (0 === strpos($requestUri, $baseDir)) {
            return $baseDir;
        }

        $truncatedRequestUri = $requestUri;

        if (false !== ($pos = strpos($requestUri, '?'))) {
            $truncatedRequestUri = substr($requestUri, 0, $pos);
        }

        $basename = basename($baseUrl);

        // No match whatsoever
        if (empty($basename) || false === strpos($truncatedRequestUri, $basename)) {
            return '';
        }

        // If using mod_rewrite or ISAPI_Rewrite strip the script filename
        // out of the base path. $pos !== 0 makes sure it is not matching a
        // value from PATH_INFO or QUERY_STRING.
        if (strlen($requestUri) >= strlen($baseUrl)
            && (false !== ($pos = strpos($requestUri, $baseUrl)) && $pos !== 0)
        ) {
            $baseUrl = substr($requestUri, 0, $pos + strlen($baseUrl));
        }

        $baseUrl = preg_replace('#/[^/]+\.php$#', '', $baseUrl);

        return $baseUrl;
    }
}
